@@ -24,35 +24,36 @@ val new_meta : unit -> metavariable
 
 (** {6 Creating a fresh evar given their type and context} *)
 
+val next_evar_name : evar_map -> intro_pattern_naming_expr -> Id.t option
+
+module VarSet :
+sig
+  type t
+  val empty : t
+  val full : t
+  val variables : Environ.env -> t
+end
+
 type naming_mode =
-  | KeepUserNameAndRenameExistingButSectionNames
-  | KeepUserNameAndRenameExistingEvenSectionNames
-  | KeepExistingNames
+  | RenameExistingBut of VarSet.t
   | FailIfConflict
-  | ProgramNaming
+  | ProgramNaming of VarSet.t
 
 val new_evar :
   ?src:Evar_kinds.t Loc.located -> ?filter:Filter.t ->
+  ?relevance:ERelevance.t ->
   ?abstract_arguments:Abstraction.t -> ?candidates:constr list ->
   ?naming:intro_pattern_naming_expr ->
   ?typeclass_candidate:bool ->
   ?principal:bool -> ?hypnaming:naming_mode ->
   env -> evar_map -> types -> evar_map * EConstr.t
 
-(** Low-level interface to create an evar.
-  @param src User-facing source for the evar
-  @param filter See {!Evd.Filter}, must be the same length as [named_context_val]
-  @param identity See {!Evd.Identity}, must be the name projection of [named_context_val]
-  @param naming A naming scheme for the evar
-  @param principal Whether the evar is the principal goal
-  @param named_context_val The context of the evar
-  @param types The type of conclusion of the evar
-*)
+(** Alias of {!Evd.new_pure_evar} *)
 val new_pure_evar :
   ?src:Evar_kinds.t Loc.located -> ?filter:Filter.t ->
-  ?identity:Identity.t ->
+  ?relevance:ERelevance.t ->
   ?abstract_arguments:Abstraction.t -> ?candidates:constr list ->
-  ?naming:intro_pattern_naming_expr ->
+  ?name:Id.t ->
   ?typeclass_candidate:bool ->
   ?principal:bool ->
   named_context_val -> evar_map -> types -> evar_map * Evar.t
@@ -64,16 +65,10 @@ val new_type_evar :
   ?naming:intro_pattern_naming_expr ->
   ?principal:bool -> ?hypnaming:naming_mode ->
   env -> evar_map -> rigid ->
-  evar_map * (constr * Sorts.t)
+  evar_map * (constr * ESorts.t)
 
 val new_Type : ?rigid:rigid -> evar_map -> evar_map * constr
 
-val make_pure_subst : evar_info -> 'a list -> (Id.t * 'a) list
-
-(** {6 Evars/Metas switching...} *)
-
-val non_instantiated : evar_map -> evar_info Evar.Map.t
-
 (** {6 Unification utils} *)
 
 (** [head_evar c] returns the head evar of [c] if any *)
@@ -89,17 +84,6 @@ val has_undefined_evars : evar_map -> constr -> bool
 val is_ground_term :  evar_map -> constr -> bool
 val is_ground_env  :  evar_map -> env -> bool
 
-(** [gather_dependent_evars evm seeds] classifies the evars in [evm]
-    as dependent_evars and goals (these may overlap). A goal is an
-    evar in [seeds] or an evar appearing in the (partial) definition
-    of a goal. A dependent evar is an evar appearing in the type
-    (hypotheses and conclusion) of a goal, or in the type or (partial)
-    definition of a dependent evar.  The value return is a map
-    associating to each dependent evar [None] if it has no (partial)
-    definition or [Some s] if [s] is the list of evars appearing in
-    its (partial) definition. *)
-val gather_dependent_evars : evar_map -> Evar.t list -> (Evar.Set.t option) Evar.Map.t
-
 (** [advance sigma g] returns [Some g'] if [g'] is undefined and is
     the current avatar of [g] (for instance [g] was changed by [clear]
     into [g']). It returns [None] if [g] has been (partially)
@@ -117,13 +101,12 @@ val reachable_from_evars : evar_map -> Evar.Set.t -> Evar.Set.t
 
 val undefined_evars_of_term : evar_map -> constr -> Evar.Set.t
 val undefined_evars_of_named_context : evar_map -> Constr.named_context -> Evar.Set.t
-val undefined_evars_of_evar_info : evar_map -> evar_info -> Evar.Set.t
 
 type undefined_evars_cache
 
 val create_undefined_evars_cache : unit -> undefined_evars_cache
 
-val filtered_undefined_evars_of_evar_info : ?cache:undefined_evars_cache -> evar_map -> evar_info -> Evar.Set.t
+val filtered_undefined_evars_of_evar_info : ?cache:undefined_evars_cache -> evar_map -> 'a evar_info -> Evar.Set.t
 
 (** [occur_evar_upto sigma k c] returns [true] if [k] appears in
     [c]. It looks up recursively in [sigma] for the value of existential
@@ -136,7 +119,7 @@ val judge_of_new_Type : evar_map -> evar_map * unsafe_judgment
 
 (***********************************************************)
 
-val create_clos_infos : env -> evar_map -> CClosure.RedFlags.reds -> CClosure.clos_infos
+val create_clos_infos : env -> evar_map -> RedFlags.reds -> CClosure.clos_infos
 
 (** [flush_and_check_evars] raise [Uninstantiated_evar] if an evar remains
     uninstantiated; [nf_evar] leaves uninstantiated evars as is *)
@@ -155,9 +138,10 @@ val nf_named_context_evar : evar_map -> Constr.named_context -> Constr.named_con
 val nf_rel_context_evar : evar_map -> rel_context -> rel_context
 val nf_env_evar : evar_map -> env -> env
 
-val nf_evar_info : evar_map -> evar_info -> evar_info
+val nf_evar_info : evar_map -> 'a evar_info -> 'a evar_info
 val nf_evar_map : evar_map -> evar_map
 val nf_evar_map_undefined : evar_map -> evar_map
+val nf_relevance : evar_map -> Sorts.relevance -> Sorts.relevance
 
 (** Presenting terms without solved evars *)
 
@@ -186,7 +170,7 @@ val finalize : ?abort_on_undefined_evars:bool -> evar_map ->
     as an evar [e] only if [e] is uninstantiated in [sigma]. Otherwise the
     value of [e] in [sigma] is (recursively) used. *)
 val kind_of_term_upto : evar_map -> Constr.constr ->
-  (Constr.constr, Constr.types, Sorts.t, Univ.Instance.t) kind_of_term
+  (Constr.constr, Constr.types, Sorts.t, UVars.Instance.t, Sorts.relevance) kind_of_term
 
 (** [eq_constr_univs_test ~evd ~extended_evd t u] tests equality of
     [t] and [u] up to existential variable instantiation and
@@ -205,14 +189,17 @@ val eq_constr_univs_test :
    constraints such that [u1 cv_pb? u2] according to [variance].
    Additionally flexible universes in irrelevant positions are unified
    if possible. Returns [Inr p] when the former is impossible. *)
-val compare_cumulative_instances : Reduction.conv_pb -> Univ.Variance.t array ->
-  Univ.Instance.t -> Univ.Instance.t -> evar_map ->
-  (evar_map, Univ.univ_inconsistency) Util.union
+val compare_cumulative_instances : Conversion.conv_pb -> UVars.Variance.t array ->
+  UVars.Instance.t -> UVars.Instance.t -> evar_map ->
+  (evar_map, UGraph.univ_inconsistency) Util.union
 
 (** We should only compare constructors at convertible types, so this
-   is only an opportunity to unify universes. *)
+    is only an opportunity to unify universes.
+
+    But what about qualities?
+*)
 val compare_constructor_instances : evar_map ->
-  Univ.Instance.t -> Univ.Instance.t -> evar_map
+  UVars.Instance.t -> UVars.Instance.t -> (evar_map, UGraph.univ_inconsistency) Util.union
 
 (** {6 Unification problems} *)
 type unification_pb = conv_pb * env * constr * constr
@@ -238,7 +225,7 @@ exception ClearDependencyError of Id.t * clear_dependency_error * GlobRef.t opti
     into an empty list. *)
 
 val restrict_evar : evar_map -> Evar.t -> Filter.t ->
-  ?src:Evar_kinds.t Loc.located -> constr list option -> evar_map * Evar.t
+  constr list option -> evar_map * Evar.t
 
 val clear_hyps_in_evi : env -> evar_map -> named_context_val -> types ->
   Id.Set.t -> evar_map * named_context_val * types
@@ -246,20 +233,30 @@ val clear_hyps_in_evi : env -> evar_map -> named_context_val -> types ->
 val clear_hyps2_in_evi : env -> evar_map -> named_context_val -> types -> types ->
   Id.Set.t -> evar_map * named_context_val * types * types
 
+val check_and_clear_in_constr
+  : Environ.env
+  -> Evd.evar_map
+  -> clear_dependency_error
+  -> Names.Id.Set.t
+  -> EConstr.constr
+  -> Evd.evar_map
+
 type csubst
 
 val empty_csubst : csubst
-val csubst_subst : csubst -> constr -> constr
+val csubst_subst : Evd.evar_map -> csubst -> constr -> constr
 
 type ext_named_context =
   csubst * Id.Set.t * named_context_val
 
-val push_rel_decl_to_named_context : ?hypnaming:naming_mode ->
+val default_ext_instance : ext_named_context -> constr SList.t
+
+val push_rel_decl_to_named_context : hypnaming:naming_mode ->
   evar_map -> rel_declaration -> ext_named_context -> ext_named_context
 
-val push_rel_context_to_named_context : ?hypnaming:naming_mode ->
+val push_rel_context_to_named_context : hypnaming:naming_mode ->
   Environ.env -> evar_map -> types ->
-  named_context_val * types * constr list * csubst
+  named_context_val * types * constr SList.t * csubst
 
 val generalize_evar_over_rels : evar_map -> existential -> types * constr list
 
